@@ -5,12 +5,12 @@ from upander import UploaderDownloader
 from upander import UploaderDownloaderResponse
 
 import requests 
-from urllib.parse import urlparse
-from typing import Optional, IO
-from http.cookiejar import MozillaCookieJar
+from urllib.parse import urlparse, quote
+from typing import Optional, IO,  List, TypedDict, Literal
 import datetime
 import re
 import time
+import dataclasses
 
 class ファイルなう(Uploader):
   UPLOAD_FILE_SIZE_LIMIT = 8 * 1024 * 1024 * 1024
@@ -34,7 +34,65 @@ class ファイルなう(Uploader):
   }
   DOMEIN = '.kuku.lu'
 
-  def SetTimelimit(url: str, cookies: dict, timelimit: str, session: requests.Session = None) -> requests.Response:
+  def DeleteList(url: str, cookies: dict, delete_only_list=False, session: requests.Session = None) -> requests.Response:
+    if delete_only_list:
+      return ファイルなう.Delete(url, cookies, session)
+
+    __hash = url.split("/")[-1]
+    headers = {
+        'authority': 'd.kuku.lu',
+        'accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.7',
+        'accept-language': 'ja',
+        'referer': 'https://d.kuku.lu'+__hash,
+        'sec-ch-ua': '"Chromium";v="110", "Not A(Brand";v="24", "Microsoft Edge";v="110"',
+        'sec-ch-ua-mobile': '?0',
+        'sec-ch-ua-platform': '"Windows"',
+        'sec-fetch-dest': 'document',
+        'sec-fetch-mode': 'navigate',
+        'sec-fetch-site': 'same-origin',
+        'sec-fetch-user': '?1',
+        'upgrade-insecure-requests': '1',
+        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36 Edg/110.0.1587.57',
+    }
+    params = {
+        'action': 'delete',
+        'delete_linkfile': '1',
+        'hash': __hash,
+    }
+    return requests.get('https://d.kuku.lu/view.php', params=params, cookies=cookies, headers=headers)
+
+  def RegList(url_list: list, cookies: dict, password: str = None, session: requests.Session = None) -> requests.Response:
+    __hash_list = [url.split("/")[-1] for url in url_list]
+    headers = {
+        'authority': 'd.kuku.lu',
+        'accept': '*/*',
+        'accept-language': 'ja',
+        'content-type': 'application/x-www-form-urlencoded; charset=UTF-8',
+        'origin': 'https://d.kuku.lu',
+        'referer': 'https://d.kuku.lu/upload.php?uuid={}&t={}'.format(cookies["cookie_uid"], time),
+        'sec-ch-ua': '"Chromium";v="110", "Not A(Brand";v="24", "Microsoft Edge";v="110"',
+        'sec-ch-ua-mobile': '?0',
+        'sec-ch-ua-platform': '"Windows"',
+        'sec-fetch-dest': 'empty',
+        'sec-fetch-mode': 'cors',
+        'sec-fetch-site': 'same-origin',
+        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/110.0.0.0 Safari/537.36 Edg/110.0.1587.57',
+        'x-requested-with': 'XMLHttpRequest',
+    }
+    data = '&action=reglist&ajax=1&uuid={}&link_hash={}'.format(cookies["cookie_uid"], "%2C".join(__hash_list))
+    if session is None:
+      import requests as _http
+    else:
+      _http = session
+    response = _http.post('https://d.kuku.lu/list.php', cookies=cookies, headers=headers, data=data)
+    if password:
+      try:
+        print(ファイルなう.SetDownloadPassWord("https://d.kuku.lu/"+response.json()["hash"], cookies, password, session).text)
+      except Exception as e:
+        print(e, response.text)
+    return response
+
+  def SetTimelimit(url: str, cookies: dict, timelimit: Literal["10M", "30M", "01H", "02H", "03H", "06H", "12H", "01D", "02D", "03D", "05D", "07D", "10D", "15D", "20D", "30D"], session: requests.Session = None) -> requests.Response:
     if ファイルなう.TIME_LIMIT.get(timelimit, None) is None:
       raise ValueError("timelimit に指定できる値は「'" + "','".join(list(ファイルなう.TIME_LIMIT.keys())) + "'」です。")
     __hash = url.split("/")[-1]
@@ -211,7 +269,7 @@ class ファイルなう(Uploader):
       cookies = response.cookies
       for cookie_domein in cookies.list_domains():
         self.__cookies[cookie_domein] = cookies.get_dict(cookie_domein)
-      self.__cookies[ファイルなう.DOMEIN]["cookie_dlpassword"] = __hash+"%40"+self.__password
+      self.__cookies[ファイルなう.DOMEIN]["cookie_dlpassword"] = (__hash+"%40"+quote(self.__password))
       return response
 
     def __regist_cookie_uid(self, session: requests.Session):
@@ -236,51 +294,72 @@ class ファイルなう(Uploader):
       def content(self) -> bytes   : return self.__content
       def cookies(self) -> dict    : return self.__cookies
 
-
   class Poster(UploaderPoster):
-    def __init__(self, file: IO, cookies: dict=None, password: str = None, timelimit: str = None):
-      self.__file = file
+    PosterRunAllResponse = TypedDict('PosterRunAllResponse', {'uploaded_files_response': List[UploaderPosterResponse], 'reglist_response': Optional[requests.Response], "cookies": dict})
+
+    def __init__(self, settings_list: list, cookies: dict=None):
+      self.__settings_list = settings_list
       if not(cookies is None):
         self.__cookies = cookies
       else:
         self.__cookies = {}
-      self.__password = password
-      if not(timelimit is None) and ファイルなう.TIME_LIMIT.get(timelimit, None) is None:
-        raise ValueError("timelimit に指定できる値は「'" + "','".join(list(ファイルなう.TIME_LIMIT.keys())) + "'」です。")
-      self.__timelimit = timelimit
+      self.__upload_cnt = 0
 
     def run(self):
+      """ファイルのアップロードを行います。
+      インスタンス作成時のsettings_list引数に従い処理を行います。一度の呼び出しで、１つのファイルのみをアップロードします。 settings_listの長さが１でない場合、１度の呼び出しで最初の要素から順に処理を行っていきます。 すべてのファイルをアップロードし終えた時に呼び出すと、Noneを返します。
+      """
       with requests.Session() as session:
         if self.__cookies.get(ファイルなう.DOMEIN, {}).get("cookie_uid", None) is None:
           self.__regist_cookie_uid(session)
 
-        response = self.__getUploadServerMulti(session)
-        if not( response.json().get("result") == "OK" ) : raise Exception()
+        if len(self.__settings_list) == self.__upload_cnt:
+          return None
 
-        upload_response = self.__upload(session, response.json()["servers"][0])
+        settings = self.__settings_list[self.__upload_cnt]
+        response = self.__getUploadServerMulti(session)
+        if not( response.json().get("result") == "OK" ) : raise Exception(response.text)
+
+        upload_response = self.__upload(session, response.json()["servers"][0], settings.file)
         upload_response = ファイルなう.Poster.Response(upload_response, self.__cookies)
 
-        if self.__password:
+        if settings.password:
           time.sleep(5)
-          response = self.__setDownloadPassWord(session, upload_response.dl_url())
-        if self.__timelimit:
+          response = self.__setDownloadPassWord(session, upload_response.dl_url(), settings.password)
+        if settings.timelimit:
           time.sleep(5)
-          response = self.__setTimeLimit(session, upload_response.dl_url())
-
+          response = self.__setTimeLimit(session, upload_response.dl_url(), settings.timelimit)
+        if settings.autoClose:
+          settings.file.close()
+        self.__upload_cnt = self.__upload_cnt + 1
       return upload_response
 
+    def run_all(self, isRegList=False, list_password: str = None) -> PosterRunAllResponse:
+      response_list = []
+      for i in range(self.__upload_cnt, len(self.__settings_list)):
+        response_list.append( self.run() )
+        time.sleep(10)
+      reglist_response = None
+      if isRegList:
+        reglist_response = ファイルなう.RegList([i.dl_url() for i in response_list], self.__cookies[ファイルなう.DOMEIN], list_password)
+      return {
+        "uploaded_files_response": response_list,
+        "reglist_response": reglist_response,
+        "cookies": self.__cookies
+      }
+
     def __regist_cookie_uid(self, session: requests.Session):
       cookies = session.get('https://d.kuku.lu/index.php', cookies=None).cookies
       for cookie_domein in cookies.list_domains():
         self.__cookies[cookie_domein] = cookies.get_dict(cookie_domein)
 
-    def __setTimeLimit(self, session: requests.Session, url: str) -> requests.Response:
-      return ファイルなう.SetTimelimit(url, self.__cookies[ファイルなう.DOMEIN], self.__timelimit, session)
+    def __setTimeLimit(self, session: requests.Session, url: str, timelimit: str) -> requests.Response:
+      return ファイルなう.SetTimelimit(url, self.__cookies[ファイルなう.DOMEIN], timelimit, session)
 
-    def __setDownloadPassWord(self, session: requests.Session, url:str) -> requests.Response:
-      return ファイルなう.SetDownloadPassWord(url, self.__cookies[ファイルなう.DOMEIN], self.__password, session)
+    def __setDownloadPassWord(self, session: requests.Session, url:str, password: str) -> requests.Response:
+      return ファイルなう.SetDownloadPassWord(url, self.__cookies[ファイルなう.DOMEIN], password, session)
 
-    def __upload(self, session: requests.Session, upload_server) -> requests.Response:
+    def __upload(self, session: requests.Session, upload_server, file: IO) -> requests.Response:
       headers = {
           'Accept': '*/*',
           'Accept-Language': 'ja',
@@ -299,8 +378,8 @@ class ファイルなう(Uploader):
           'ajax': (None, '1'),
           'uuid': (None, self.__cookies[ファイルなう.DOMEIN]["cookie_uid"]),
           'country': (None, 'JP'),
-          'file_1': self.__file,
-          'file_1_name': (None, self.__file.name.split("/")[-1]),
+          'file_1': file,
+          'file_1_name': (None, file.name.split("/")[-1]),
           'file_1_type': (None, 'file'),
           'filecnt': (None, '1'),
       }
@@ -330,9 +409,19 @@ class ファイルなう(Uploader):
       response = session.post('https://d.kuku.lu/index.php', cookies=self.__cookies[ファイルなう.DOMEIN], headers=headers, data=data)
       return response
 
+    @dataclasses.dataclass(frozen=True)
+    class Settings:
+      file: IO
+      password: str = None
+      timelimit: Literal["10M", "30M", "01H", "02H", "03H", "06H", "12H", "01D", "02D", "03D", "05D", "07D", "10D", "15D", "20D", "30D"] = None
+      autoClose: bool = True
+      def __post_init__(self):
+        if not(self.timelimit is None) and ファイルなう.TIME_LIMIT.get(self.timelimit, None) is None:
+          raise ValueError("timelimit に指定できる値は「'" + "','".join(list(ファイルなう.TIME_LIMIT.keys())) + "'」です。")
+
     class Response(UploaderPosterResponse):
       def __init__(self, response: requests.Response, cookies: dict):
-        if not( response.text[:response.text.find(":")] == "OK" ) : raise Exception()
+        if not( response.text[:response.text.find(":")] == "OK" ) : raise Exception(response.text)
         self.__dl_url = response.text[response.text.find(":")+1:]
         self.__dl_file_name = ""
         self.__status = response.status_code
